/***************************************************************************
                          kplayernodeview.cpp
                          -------------------
    begin                : Mon Apr 18 2005
    copyright            : (C) 2005-2008 by kiriuja
    email                : http://kplayer.sourceforge.net/email.html
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <kiconloader.h>
#include <kinputdialog.h>
#include <klineedit.h>
#include <klocale.h>
#include <qaction.h>
#include <qcursor.h>
#include <qevent.h>
#include <qheaderview.h>
#include <qlabel.h>
#include <qmenu.h>
#include <qscrollbar.h>
#include <qtimer.h>

#ifdef DEBUG
#define DEBUG_KPLAYER_NODEVIEW
#define DEBUG_KPLAYER_NODEMODEL
#endif

#include "kplayernodeview.h"
#include "kplayernodeview.moc"
#include "kplayernodeaction.h"
#include "kplayerplaylist.h"
#include "kplayerpropertiesdialog.h"
#include "kplayersettings.h"

KPlayerNodeNameValidator::KPlayerNodeNameValidator (KPlayerContainerNode* node, QObject* parent)
  : QValidator (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating node name validator\n";
  kdDebugTime() << " URL    " << node -> url().url() << "\n";
#endif
  m_node = node;
  node -> reference();
  node -> populateGroups();
}

KPlayerNodeNameValidator::~KPlayerNodeNameValidator()
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying node name validator\n";
  kdDebugTime() << " URL    " << m_node -> url().url() << "\n";
#endif
  m_node -> vacateGroups();
  m_node -> release();
}

QValidator::State KPlayerNodeNameValidator::validate (QString& input, int&) const
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeNameValidator::validate\n";
  kdDebugTime() << " Input  " << input << "\n";
#endif
  if ( ! input.isEmpty() && input.indexOf ('/') < 0 )
  {
    KPlayerNode* node = m_node -> nodeById (input);
#ifdef DEBUG_KPLAYER_NODEVIEW
    if ( node )
      kdDebugTime() << " Node   " << node -> url().url() << " container " << node -> isContainer() << "\n";
#endif
    if ( ! node || ! node -> isContainer() )
      return QValidator::Acceptable;
  }
  return QValidator::Intermediate;
}

KPlayerPropertiesDevice::KPlayerPropertiesDevice (QWidget* parent)
  : QFrame (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating device properties page\n";
#endif
  m_node = KPlayerNode::root() -> getNodeByUrl (KUrl ("kplayer:/devices"));
  m_node -> reference();
  m_node -> populateGroups();
  setupUi (this);
  QString list (KPlayerTVProperties::channelListFromCountry());
  for ( uint i = 0; i < channellistcount; i ++ )
  {
    c_channel_list -> addItem (i18n(channellists[i].name));
    if ( channellists[i].id == list )
      c_channel_list -> setCurrentIndex (i);
  }
  pathChanged (c_path -> text());
  typeChanged (c_type -> currentIndex());
  c_driver -> setCurrentIndex (2);
}

KPlayerPropertiesDevice::~KPlayerPropertiesDevice()
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying device properties page\n";
#endif
  m_node -> vacateGroups();
  m_node -> release();
}

void KPlayerPropertiesDevice::pathChanged (const QString& path)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerPropertiesDevice::pathChanged\n";
  kdDebugTime() << " Path   " << path << "\n";
#endif
  QString fullpath (path), slash ("/");
  if ( ! path.startsWith (slash) )
    fullpath.prepend (slash);
  bool enable = ! c_name -> text().isEmpty() && fullpath != slash && ! m_node -> nodeById (fullpath);
  parent() -> enableButton (KDialog::Ok, enable);
}

void KPlayerPropertiesDevice::typeChanged (int index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerPropertiesDevice::typeChanged\n";
  kdDebugTime() << " Type   " << c_type -> itemText (index) << "\n";
#endif
  if ( index == 2 )
  {
    l_channel_list -> show();
    c_channel_list -> show();
    l_driver -> show();
    c_driver -> show();
  }
  else
  {
    l_channel_list -> hide();
    c_channel_list -> hide();
    l_driver -> hide();
    c_driver -> hide();
  }
  if ( index == 3 )
  {
    l_channel_file -> show();
    c_channel_file -> show();
  }
  else
  {
    l_channel_file -> hide();
    c_channel_file -> hide();
  }
}

void KPlayerPropertiesDevice::addDevice (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerPropertiesDevice::addDevice\n";
#endif
  QString fullpath (c_path -> text()), slash ("/");
  if ( ! fullpath.startsWith (slash) )
    fullpath.prepend (slash);
  KPlayerDeviceProperties* media = KPlayerMedia::deviceProperties (m_node -> url (fullpath));
  media -> setName (c_name -> text());
  int index = c_type -> currentIndex();
  media -> setType (index == 1 ? "DVD" : index == 2 ? "TV" : index == 3 ? "DVB" : "CD");
  if ( index == 2 )
  {
    ((KPlayerTunerProperties*) media) -> setChannelList (channellists[c_channel_list -> currentIndex()].id);
    int driver = c_driver -> currentIndex();
    ((KPlayerTVProperties*) media) -> setInputDriver (driver == 0 ? "bsdbt848" : driver == 1 ? "v4l" : "v4l2");
  }
  else if ( index == 3 && ! c_channel_file -> text().isEmpty() )
    ((KPlayerTunerProperties*) media) -> setChannelList (c_channel_file -> text());
  media -> commit();
  QStringList list;
  list.append (fullpath);
  m_node -> addedBranches (list);
  KPlayerMedia::release (media);
}

KPlayerDeviceDialog::KPlayerDeviceDialog (QWidget* parent)
  : KDialog (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating device dialog\n";
#endif
  setCaption (i18n("Add device"));
  setButtons (KDialog::Help | KDialog::Ok | KDialog::Cancel);
  setMainWidget (new KPlayerPropertiesDevice (this));
}

KPlayerDeviceDialog::~KPlayerDeviceDialog (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying device dialog\n";
#endif
}

KPlayerHistory::KPlayerHistory (QObject* parent)
  : QObject (parent)
{
  m_current = 0;
  m_navigating = false;
}

void KPlayerHistory::goBack (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerHistory::goBack\n";
#endif
  KPlayerContainerNode* node = 0;
  while ( ! node && m_current > 0 )
  {
    -- m_current;
    node = KPlayerNode::root() -> getNodeByUrl (m_entries.at (m_current).m_url);
    if ( node )
    {
      m_navigating = true;
      emit navigate (node);
      m_navigating = false;
    }
    else
    {
      m_entries.removeAt (m_current);
      emit updated();
    }
  }
#ifdef DEBUG_KPLAYER_NODEVIEW
  if ( m_current < m_entries.count() )
    kdDebugTime() << "Current URL " << m_entries.at (m_current).m_url.url() << "\n";
#endif
}

void KPlayerHistory::goForward (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerHistory::goForward\n";
#endif
  KPlayerContainerNode* node = 0;
  while ( ! node && m_current < m_entries.count() )
  {
    ++ m_current;
    if ( m_current < m_entries.count() )
    {
      node = KPlayerNode::root() -> getNodeByUrl (m_entries.at (m_current).m_url);
      if ( node )
      {
        m_navigating = true;
        emit navigate (node);
        m_navigating = false;
      }
      else
      {
        m_entries.removeAt (m_current);
        emit updated();
      }
    }
  }
#ifdef DEBUG_KPLAYER_NODEVIEW
  if ( m_current < m_entries.count() )
    kdDebugTime() << "Current URL " << m_entries.at (m_current).m_url.url() << "\n";
#endif
}

void KPlayerHistory::goTo (int index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerHistory::goTo\n";
  kdDebugTime() << " Index  " << index << "\n";
  kdDebugTime() << " URL    " << m_entries.at (index).m_url.url() << "\n";
#endif
  if ( index != m_current )
  {
    KPlayerContainerNode* node = KPlayerNode::root() -> getNodeByUrl (m_entries.at (index).m_url);
    if ( node )
    {
      m_current = index;
      m_navigating = true;
      emit navigate (node);
      m_navigating = false;
    }
    else
    {
      m_entries.removeAt (index);
      emit updated();
    }
  }
#ifdef DEBUG_KPLAYER_NODEVIEW
  if ( m_current < m_entries.count() )
    kdDebugTime() << "Current URL " << m_entries.at (m_current).m_url.url() << "\n";
#endif
}

void KPlayerHistory::apply (KPlayerListView* view)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerHistory::apply\n";
#endif
  if ( m_navigating )
  {
    if ( m_current < m_entries.count() )
      ++ m_current;
    while ( m_current < m_entries.count() )
      m_entries.removeAt (m_current);
    KPlayerNode* node = view -> model() -> nodeFromIndex (view -> rootIndex());
    KPlayerHistoryEntry entry;
    entry.m_url = node -> url();
    entry.m_name = node -> name();
    m_entries.append (entry);
    emit updated();
#ifdef DEBUG_KPLAYER_NODEVIEW
    kdDebugTime() << "Current URL " << m_entries.at (m_current).m_url.url() << "\n";
#endif
  }
  else if ( m_current < m_entries.count() )
  {
    const KPlayerHistoryEntry& entry = m_entries.at (m_current);
#ifdef DEBUG_KPLAYER_NODEVIEW
    kdDebugTime() << " Expanded " << entry.m_expanded.count() << "\n";
    kdDebugTime() << " Selected " << entry.m_selected.count() << "\n";
    kdDebugTime() << " Current container " << entry.m_current.url() << "\n";
    kdDebugTime() << " Current ID " << entry.m_current_id << "\n";
    kdDebugTime() << " Position " << entry.m_position.x() << "x" << entry.m_position.y() << "\n";
#endif
    KPlayerNodeModel* model = view -> model();
    QItemSelectionModel* selection = view -> selectionModel();
    if ( view -> horizontalScrollBar() )
      view -> horizontalScrollBar() -> setValue (entry.m_position.x());
    if ( view -> verticalScrollBar() )
      view -> verticalScrollBar() -> setValue (entry.m_position.y());
    KPlayerContainerNode* container;
    KPlayerNode* node;
    KUrl::List::ConstIterator iterator (entry.m_expanded.constBegin());
    while ( iterator != entry.m_expanded.constEnd() )
    {
      node = KPlayerNode::root() -> getNodeByUrl (*iterator);
      if ( node )
        view -> setExpanded (model -> indexForNode (node), true);
      ++ iterator;
    }
    QStringList::ConstIterator iditerator (entry.m_selected_ids.constBegin());
    iterator = entry.m_selected.constBegin();
    while ( iterator != entry.m_selected.constEnd() && iditerator != entry.m_selected_ids.constEnd() )
    {
      container = KPlayerNode::root() -> getNodeByUrl (*iterator);
      if ( container )
      {
        node = container -> nodeById (*iditerator);
        if ( node )
          selection -> select (model -> indexForNode (node), QItemSelectionModel::Select);
      }
      ++ iditerator;
      ++ iterator;
    }
    if ( ! entry.m_current.isEmpty() && ! entry.m_current_id.isNull() )
    {
      container = KPlayerNode::root() -> getNodeByUrl (entry.m_current);
      if ( container )
      {
        node = container -> nodeById (entry.m_current_id);
        if ( node )
          selection -> setCurrentIndex (model -> indexForNode (node), QItemSelectionModel::Current);
      }
    }
  }
}

void KPlayerHistory::store (KPlayerListView* view)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerHistory::store\n";
#endif
  if ( m_current >= m_entries.count() )
    return;
  KPlayerHistoryEntry& entry = m_entries [m_current];
  KPlayerNodeModel* model = view -> model();
  QItemSelectionModel* selection = view -> selectionModel();
  QModelIndex index;
  KPlayerNode* node;
  entry.m_expanded.clear();
  entry.m_selected.clear();
  entry.m_selected_ids.clear();
  for ( index = view -> topIndex(); index.isValid(); index = view -> indexBelow (index) )
  {
    node = model -> nodeFromIndex (index);
    if ( selection -> isSelected (index) )
    {
      entry.m_selected.append (node -> parent() -> url());
      entry.m_selected_ids.append (node -> id());
    }
    if ( view -> isExpanded (index) && node -> isContainer() )
      entry.m_expanded.append (node -> url());
  }
  node = model -> nodeFromIndex (selection -> currentIndex());
  entry.m_current = node -> parent() ? node -> parent() -> url() : KUrl();
  entry.m_current_id = node -> parent() ? node -> id() : QString::null;
  entry.m_position.setX (view -> horizontalScrollBar() ? view -> horizontalScrollBar() -> value() : 0);
  entry.m_position.setY (view -> verticalScrollBar() ? view -> verticalScrollBar() -> value() : 0);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Expanded " << entry.m_expanded.count() << "\n";
  kdDebugTime() << " Selected " << entry.m_selected.count() << "\n";
  kdDebugTime() << " Current container " << entry.m_current.url() << "\n";
  kdDebugTime() << " Current ID " << entry.m_current_id << "\n";
  kdDebugTime() << " Position " << entry.m_position.x() << "x" << entry.m_position.y() << "\n";
#endif
}

KPlayerHistoryActionList::KPlayerHistoryActionList (const KPlayerHistory* history, const KLocalizedString& text,
  const KLocalizedString& status, const KLocalizedString& whatsthis, QObject* parent, const QString& name)
  : KPlayerActionList (text, status, whatsthis, parent, name), m_history (history)
{
#ifdef DEBUG_KPLAYER_ACTIONLIST
  kdDebugTime() << "Creating history action list\n";
#endif
}

KPlayerHistoryActionList::~KPlayerHistoryActionList()
{
#ifdef DEBUG_KPLAYER_ACTIONLIST
  kdDebugTime() << "Destroying history action list\n";
#endif
}

void KPlayerHistoryActionList::update (void)
{
#ifdef DEBUG_KPLAYER_ACTIONLIST
  kdDebugTime() << "KPlayerHistoryActionList::update\n";
#endif
  unplug();
  m_bottom = m_history -> entries().count();
  if ( m_bottom > 1 )
  {
    int limit = KPlayerEngine::engine() -> configuration() -> recentMenuSize();
    int halflimit = limit / 2;
    int count = 0;
    QList<KPlayerHistoryEntry>::ConstIterator iterator (m_history -> entries().end());
    do
    {
      -- iterator;
      -- m_bottom;
      if ( actions().count() == limit )
        m_actions.removeFirst();
      KToggleAction* action = new KToggleAction (this);
      connect (action, SIGNAL (triggered()), SLOT (actionActivated()));
      action -> setText ((*iterator).m_name);
      if ( m_bottom == m_history -> currentIndex() )
      {
        action -> setChecked (true);
        ++ count;
      }
      else if ( count )
        ++ count;
      updateAction (action);
      m_actions.append (action);
    }
    while ( iterator != m_history -> entries().begin() && (count < halflimit || actions().count() < limit) );
    plug();
  }
}

void KPlayerHistoryActionList::actionActivated (QAction*, int index)
{
#ifdef DEBUG_KPLAYER_ACTIONLIST
  kdDebugTime() << "KPlayerHistoryActionList::actionActivated\n";
  kdDebugTime() << " Index  " << index << "\n";
#endif
  emit activated (m_bottom + actions().count() - index - 1);
}

KPlayerNodeAttributes::KPlayerNodeAttributes (QObject* parent)
  : QObject (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating node attributes\n";
#endif
  KConfigGroup group (config() -> group ("Multimedia Library"));
  m_attribute_order = group.hasKey ("Column Order") ? group.readEntry ("Column Order", QStringList())
    : KPlayerMedia::defaultOrder();
  QStringList::ConstIterator iterator (attributeOrder().begin());
  while ( iterator != attributeOrder().end() )
  {
    const QString& name (*iterator);
    if ( ! name.isEmpty() && group.hasKey ("Show Column " + name) )
      KPlayerMedia::info (name) -> setShow (group.readEntry ("Show Column " + name, false));
    ++ iterator;
  }
}

KPlayerNodeAttributes::~KPlayerNodeAttributes()
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying node attributes\n";
#endif
}

void KPlayerNodeAttributes::initialize (bool directory)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeAttributes::initialize\n";
#endif
  m_directory = directory;
  m_attribute_names.clear();
  m_available_attributes.clear();
  m_attribute_states.clear();
  m_column_states.clear();
  m_editable_attributes.clear();
  m_attribute_counts.clear();
}

void KPlayerNodeAttributes::update (const KPlayerPropertyCounts& added, const KPlayerPropertyCounts& removed)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeAttributes::update\n";
#endif
  m_attribute_counts.add (added);
  m_attribute_counts.subtract (removed);
  KPlayerPropertyCounts::ConstIterator it = attributeCounts().begin();
  while ( it != attributeCounts().end() )
  {
    const QString& name (it.key());
    if ( KPlayerMedia::info (name) -> canShow() && ! attributeOrder().contains (name) )
      m_attribute_order.insert (m_attribute_order.indexOf (""), name);
    ++ it;
  }
  QStringList::ConstIterator iterator (attributeOrder().begin());
  if ( attributeNames().isEmpty() )
    insert (m_attribute_names, *iterator);
  ++ iterator;
  while ( iterator != attributeOrder().end() )
  {
    const QString& name (*iterator);
    if ( ! name.isEmpty() )
    {
      KPlayerPropertyInfo* info = KPlayerMedia::info (name);
      if ( ! attributeCounts().contains (name) || m_directory && name == "Path" )
      {
        m_available_attributes.removeAll (name);
        m_attribute_states.insert (name, false);
      }
      else
      {
        if ( ! availableAttributes().contains (name) )
          insert (m_available_attributes, name);
        m_attribute_states.insert (name, info -> show() && attributeCounts().contains (name) || showColumn (name));
      }
      if ( ! info -> canEdit() )
        m_editable_attributes.removeAll (name);
      else if ( ! editableAttributes().contains (name) )
        insert (m_editable_attributes, name);
      if ( ! attributeNames().contains (name) && attributeStates() [name] )
        insert (m_attribute_names, name);
      else if ( attributeNames().contains (name) && ! attributeStates() [name] )
        m_attribute_names.removeAll (name);
    }
    ++ iterator;
  }
  emit updated();
}

void KPlayerNodeAttributes::insert (QStringList& list, const QString& name)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeAttributes::insert\n";
  kdDebugTime() << " Name   " << name << "\n";
#endif
  list.removeAll (name);
  QStringList::ConstIterator iterator (attributeOrder().begin());
  QStringList::Iterator it (list.begin());
  while ( it != list.end() && iterator != attributeOrder().end() && *iterator != name )
  {
    if ( *it == *iterator )
      ++ it;
    ++ iterator;
  }
  list.insert (it, name);
}

void KPlayerNodeAttributes::toggleState (const QString& name)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeAttributes::toggleState\n";
  kdDebugTime() << " Name   " << name << "\n";
#endif
  KPlayerPropertyInfo* info = KPlayerMedia::info (name);
  info -> setShow (! attributeStates() [name]);
  m_attribute_states.insert (name, info -> show());
  m_column_states.insert (name, info -> show());
  if ( info -> show() )
    insert (m_attribute_names, name);
  else
    m_attribute_names.removeAll (name);
  emit updated();
  if ( info -> show() == info -> showByDefault() )
    config() -> group ("Multimedia Library").deleteEntry ("Show Column " + name);
  else
    config() -> group ("Multimedia Library").writeEntry ("Show Column " + name, info -> show());
}

int KPlayerNodeAttributes::editableIndex (int index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeAttributes::editableIndex\n";
  kdDebugTime() << " Index  " << index << "\n";
#endif
  const QString& name (editableAttributes() [index]);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Name   " << name << "\n";
#endif
  index = attributeNames().indexOf (name);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Index  " << index << "\n";
#endif
  if ( index < 0 )
  {
    index = availableAttributes().indexOf (name);
#ifdef DEBUG_KPLAYER_NODEVIEW
    kdDebugTime() << " Index  " << index << "\n";
#endif
    if ( index < 0 )
    {
      insert (m_available_attributes, name);
      m_attribute_states.insert (name, false);
      index = availableAttributes().indexOf (name);
#ifdef DEBUG_KPLAYER_NODEVIEW
      kdDebugTime() << " Index  " << index << "\n";
#endif
    }
    emit updated();
    index = attributeNames().indexOf (name);
#ifdef DEBUG_KPLAYER_NODEVIEW
    kdDebugTime() << " Index  " << index << "\n";
#endif
  }
  return index;
}

void KPlayerNodeAttributes::move (int from, int to)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeAttributes::move\n";
  kdDebugTime() << " From   " << from << "\n";
  kdDebugTime() << " To     " << to << "\n";
#endif
  QString name (attributeNames() [from]);
  m_attribute_order.removeAll (name);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Name   " << name << "\n";
#endif
  m_attribute_order.insert (to, name);
  insert (m_attribute_names, name);
  KPlayerPropertyInfo* info = KPlayerMedia::info (name);
  insert (m_available_attributes, name);
  if ( info -> canEdit() )
    insert (m_editable_attributes, name);
  emit updated();
  KConfigGroup group (config() -> group ("Multimedia Library"));
  if ( attributeOrder() == KPlayerMedia::defaultOrder() )
    group.deleteEntry ("Column Order");
  else
    group.writeEntry ("Column Order", attributeOrder());
}

KPlayerNodeModel::KPlayerNodeModel (QObject* parent)
  : QAbstractItemModel (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating node model\n";
#endif
}

KPlayerNodeModel::~KPlayerNodeModel()
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying node model\n";
#endif
}

void KPlayerNodeModel::initialize (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::initialize\n";
#endif
  populate (KPlayerNode::root());
}

void KPlayerNodeModel::terminate (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::terminate\n";
#endif
  QList<KPlayerContainerNode*> populated (m_populated);
  QList<KPlayerContainerNode*>::ConstIterator iterator (populated.constBegin());
  while ( iterator != populated.constEnd() )
  {
#ifdef DEBUG_KPLAYER_NODEVIEW
    kdDebugTime() << " Vacating " << (*iterator) -> url() << "\n";
#endif
    vacate (*iterator);
    ++ iterator;
  }
}

void KPlayerNodeModel::connectNode (KPlayerContainerNode* node)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::connectNode\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  connect (node, SIGNAL (nodesAdded (KPlayerContainerNode*, const KPlayerNodeList&, KPlayerNode*)),
    SLOT (added (KPlayerContainerNode*, const KPlayerNodeList&, KPlayerNode*)));
  connect (node, SIGNAL (nodesRemoved (KPlayerContainerNode*, const KPlayerNodeList&)),
    SLOT (removed (KPlayerContainerNode*, const KPlayerNodeList&)));
  connect (node, SIGNAL (nodeUpdated (KPlayerContainerNode*, KPlayerNode*)),
    SLOT (updated (KPlayerContainerNode*, KPlayerNode*)));
}

void KPlayerNodeModel::disconnectNode (KPlayerContainerNode* node)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::disconnectNode\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  disconnect (node, SIGNAL (nodesAdded (KPlayerContainerNode*, const KPlayerNodeList&, KPlayerNode*)),
    this, SLOT (added (KPlayerContainerNode*, const KPlayerNodeList&, KPlayerNode*)));
  disconnect (node, SIGNAL (nodesRemoved (KPlayerContainerNode*, const KPlayerNodeList&)),
    this, SLOT (removed (KPlayerContainerNode*, const KPlayerNodeList&)));
  disconnect (node, SIGNAL (nodeUpdated (KPlayerContainerNode*, KPlayerNode*)),
    this, SLOT (updated (KPlayerContainerNode*, KPlayerNode*)));
}

void KPlayerNodeModel::added (KPlayerContainerNode* container, const KPlayerNodeList& nodes, KPlayerNode* after)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::added\n";
#endif
  emit layoutChanged();
  if ( expandable() && parent() -> isExpanded (indexForNode (container)) )
    for ( KPlayerNodeList::ConstIterator iterator (nodes.begin()); iterator != nodes.end(); ++ iterator )
      if ( (*iterator) -> isContainer() )
        connectNode ((KPlayerContainerNode*) *iterator);
}

void KPlayerNodeModel::removed (KPlayerContainerNode* container, const KPlayerNodeList& nodes)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::removed\n";
#endif
  if ( expandable() && parent() -> isExpanded (indexForNode (container)) )
    for ( KPlayerNodeList::ConstIterator iterator (nodes.begin()); iterator != nodes.end(); ++ iterator )
      if ( (*iterator) -> isContainer() )
        disconnectNode ((KPlayerContainerNode*) *iterator);
  emit layoutChanged();
}

void KPlayerNodeModel::updated (KPlayerContainerNode*, KPlayerNode* node)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::updated\n";
#endif
  emit layoutChanged();
}

void KPlayerNodeModel::newRootIndex (const QModelIndex& index, const QModelIndex& old)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::newRootIndex\n";
#endif
  KPlayerContainerNode* previous = (KPlayerContainerNode*) nodeFromIndex (old);
  KPlayerContainerNode* node = (KPlayerContainerNode*) nodeFromIndex (index);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Node     " << node -> url() << "\n";
  kdDebugTime() << " Previous " << previous -> url() << "\n";
#endif
  if ( node != previous )
  {
    disconnectNode (previous);
    vacateIndex (old);
    m_expandable = node -> isPlaylist() || ! node -> parent();
#ifdef DEBUG_KPLAYER_NODEVIEW
    kdDebugTime() << " Expandable " << m_expandable << "\n";
#endif
    populate (node);
    connectNode (node);
  }
}

/*void KPlayerNodeModel::expanded (const QModelIndex& index)
{
}*/

void KPlayerNodeModel::populate (KPlayerContainerNode* node)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::populate\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  if ( ! m_populated.contains (node) )
  {
    m_populated.append (node);
    node -> reference();
    populateNode (node);
    node -> applyOrder();
    if ( expandable() )
      for ( KPlayerNodeList::ConstIterator iterator (node -> nodes().begin()); iterator != node -> nodes().end(); ++ iterator )
        if ( (*iterator) -> isContainer() )
          connectNode ((KPlayerContainerNode*) *iterator);
  }
}

void KPlayerNodeModel::vacate (KPlayerContainerNode* node)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::vacate\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  if ( m_populated.contains (node) )
  {
    m_populated.removeAll (node);
    if ( expandable() )
      for ( KPlayerNodeList::ConstIterator iterator (node -> nodes().begin()); iterator != node -> nodes().end(); ++ iterator )
        if ( (*iterator) -> isContainer() )
          disconnectNode ((KPlayerContainerNode*) *iterator);
    vacateNode (node);
    node -> release();
  }
}

void KPlayerNodeModel::populateIndex (const QModelIndex& index)
{
  KPlayerContainerNode* node = (KPlayerContainerNode*) nodeFromIndex (index);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::populateIndex\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  populate (node);
}

void KPlayerNodeModel::vacateIndex (const QModelIndex& index)
{
  KPlayerContainerNode* node = (KPlayerContainerNode*) nodeFromIndex (index);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::vacateIndex\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  QList<KPlayerContainerNode*> populated (m_populated);
  QList<KPlayerContainerNode*>::ConstIterator iterator (populated.constBegin());
  while ( iterator != populated.constEnd() )
  {
    KPlayerContainerNode* container = *iterator;
#ifdef DEBUG_KPLAYER_NODEVIEW
    kdDebugTime() << " Populated " << container -> url() << "\n";
#endif
    if ( container == node || container -> ancestor (node) )
      vacate (container);
    ++ iterator;
  }
}

bool KPlayerNodeModel::ancestor (const QModelIndex& index, const QModelIndex& container) const
{
  return nodeFromIndex (index) -> ancestor ((KPlayerContainerNode*) nodeFromIndex (container));
}

QModelIndex KPlayerNodeModel::indexForNode (KPlayerNode* node) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerNodeModel::indexForNode\n";
#endif
  if ( node )
  {
    KPlayerContainerNode* parent = node -> parent();
#ifdef DEBUG_KPLAYER_NODEMODEL
    kdDebugTime() << " Node   " << node -> hexID().constData() << " " << node -> url() << "\n";
#endif
    if ( parent )
    {
      int row = nodeIndex (parent, node);
#ifdef DEBUG_KPLAYER_NODEMODEL
      kdDebugTime() << " Parent " << parent -> url() << "\n";
      kdDebugTime() << " Row    " << row << "\n";
#endif
      if ( row >= 0 )
        return createIndex (row, 0, node);
    }
  }
  return QModelIndex();
}

QModelIndex KPlayerNodeModel::index (int row, int column, const QModelIndex& parent) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerNodeModel::index\n";
  kdDebugTime() << " Row    " << row << "\n";
  kdDebugTime() << " Column " << column << "\n";
#endif
  if ( ! parent.isValid() || column >= 0 && column < columnCount (parent) && row >= 0 && parent.column() == 0 )
  {
    KPlayerNode* node = nodeFromIndex (parent);
#ifdef DEBUG_KPLAYER_NODEMODEL
    kdDebugTime() << " Parent " << node -> url() << "\n";
#endif
    if ( node -> isContainer() )
    {
      KPlayerContainerNode* container = (KPlayerContainerNode*) node;
#ifdef DEBUG_KPLAYER_NODEMODEL
      kdDebugTime() << " Count  " << container -> nodes().count() << "\n";
#endif
      node = nodeAt (container, row);
      if ( node )
      {
#ifdef DEBUG_KPLAYER_NODEMODEL
        kdDebugTime() << " URL    " << node -> url() << "\n";
#endif
        return createIndex (row, column, node);
      }
    }
  }
  return QModelIndex();
}

QModelIndex KPlayerNodeModel::parent (const QModelIndex& child) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerNodeModel::parent\n";
#endif
  return indexForNode (nodeFromIndex (child) -> parent());
}

int KPlayerNodeModel::rowCount (const QModelIndex& parent) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerNodeModel::rowCount\n";
#endif
  if ( ! parent.isValid() || parent.column() == 0 )
  {
    KPlayerNode* node = nodeFromIndex (parent);
#ifdef DEBUG_KPLAYER_NODEMODEL
    kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
    if ( node -> isContainer() )
    {
      KPlayerContainerNode* container = (KPlayerContainerNode*) node;
#ifdef DEBUG_KPLAYER_NODEMODEL
      kdDebugTime() << " Count  " << container -> nodes().count() << "\n";
#endif
      if ( ! populated (container) )
        ((KPlayerNodeModel*) this) -> populate (container);
      return nodeCount (container);
    }
  }
  return 0;
}

int KPlayerNodeModel::columnCount (const QModelIndex& parent) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerNodeModel::columnCount\n";
  kdDebugTime() << " Column " << parent.column() << "\n";
#endif
  return ! parent.isValid() || parent.column() == 0 ? attributeCount() : 0;
}

QVariant KPlayerNodeModel::data (const QModelIndex& index, int role) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerNodeModel::data\n";
  kdDebugTime() << " Role   " << role << "\n";
#endif
  KPlayerNode* node = nodeFromIndex (index);
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << " URL    " << node -> url() << "\n";
#endif
  if ( (role == Qt::DisplayRole || role == Qt::EditRole) && index.column() >= 0 && index.column() < attributeCount() )
    return nodeData (node, index.column());
  else if ( role == Qt::DecorationRole && index.column() == 0 )
    return DesktopIcon (node -> icon(), KIconLoader::SizeSmall);
  return QVariant();
}

bool KPlayerNodeModel::setData (const QModelIndex& index, const QVariant& value, int role)
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerNodeModel::setData\n";
#endif
  KPlayerNode* node = nodeFromIndex (index);
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << " URL    " << node -> url() << "\n";
#endif
  if ( role == Qt::EditRole && index.column() >= 0 && index.column() < attributeCount() )
  {
    setNodeData (node, index.column(), value.toString());
    return true;
  }
  return false;
}

QVariant KPlayerNodeModel::headerData (int column, Qt::Orientation orientation, int role) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerNodeModel::headerData\n";
  kdDebugTime() << " Column " << column << "\n";
  kdDebugTime() << " Orientation " << orientation << "\n";
  kdDebugTime() << " Role   " << role << "\n";
#endif
  if ( orientation == Qt::Horizontal && role == Qt::DisplayRole )
    return attributeLabel (column);
  return QAbstractItemModel::headerData (column, orientation, role);
}

bool KPlayerNodeModel::hasChildren (const QModelIndex& index) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerNodeModel::hasChildren\n";
  kdDebugTime() << " Column " << index.column() << "\n";
#endif
  if ( ! index.isValid() || index.column() == 0 )
  {
    KPlayerNode* node = nodeFromIndex (index);
#ifdef DEBUG_KPLAYER_NODEMODEL
    kdDebugTime() << " URL    " << node -> url() << "\n";
#endif
    if ( node -> isContainer() )
      return hasNodes ((KPlayerContainerNode*) node);
  }
  return false;
}

Qt::ItemFlags KPlayerNodeModel::flags (const QModelIndex& index) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerNodeModel::flags\n";
#endif
  Qt::ItemFlags flags = QAbstractItemModel::flags (index);
  flags |= Qt::ItemIsDragEnabled;
  flags |= Qt::ItemIsEditable;
  KPlayerNode* node = nodeFromIndex (index);
  if ( node -> isContainer() )
    flags |= Qt::ItemIsDropEnabled;
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << " Flags  " << flags << "\n";
#endif
  return flags;
}

QStringList KPlayerNodeModel::mimeTypes (void) const
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::mimeTypes\n";
#endif
  return QStringList();
}

QMimeData* KPlayerNodeModel::mimeData (const QModelIndexList& indexes) const
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::mimeData\n";
#endif
  return new QMimeData();
}

bool KPlayerNodeModel::dropMimeData (const QMimeData* data, Qt::DropAction action,
  int row, int column, const QModelIndex& parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::dropMimeData\n";
#endif
  return false;
}

Qt::DropActions KPlayerNodeModel::supportedDropActions (void) const
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeModel::supportedDropActions\n";
#endif
  return Qt::CopyAction | Qt::LinkAction | Qt::MoveAction;
}

KPlayerNodeList KPlayerNodeModel::nodesFromIndexList (const QModelIndexList& indexes) const
{
  KPlayerNodeList list;
  for ( QModelIndexList::ConstIterator iterator = indexes.begin(); iterator != indexes.end(); ++ iterator )
    list.append (nodeFromIndex (*iterator));
  return list;
}

void KPlayerNodeModel::move (const QModelIndexList& list, const QModelIndex& after)
{
  if ( ! list.isEmpty() )
  {
    KPlayerNodeList nodes = nodesFromIndexList (list);
    nodes.first() -> parent() -> move (nodes, nodeFromIndex (after));
  }
}

void KPlayerNodeModel::remove (const QModelIndexList& list)
{
  if ( ! list.isEmpty() )
  {
    KPlayerNodeList nodes = nodesFromIndexList (list);
    nodes.first() -> parent() -> remove (nodes);
  }
}

KPlayerListModel::KPlayerListModel (QObject* parent)
  : KPlayerNodeModel (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating tree model\n";
#endif
  m_attributes = new KPlayerNodeAttributes (this);
}

KPlayerListModel::~KPlayerListModel()
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying tree model\n";
#endif
}

void KPlayerListModel::initialize (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListModel::initialize\n";
#endif
  KPlayerNodeModel::initialize();
  connect (attributes(), SIGNAL (updated()), this, SLOT (updateColumns()));
}

void KPlayerListModel::terminate (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListModel::terminate\n";
#endif
  KPlayerNodeModel::terminate();
}

bool KPlayerListModel::populated (KPlayerContainerNode* node) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerListModel::populated\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  return node -> populated();
}

void KPlayerListModel::populateNode (KPlayerContainerNode* node)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListModel::populateNode\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  node -> populate();
  attributes() -> update (node -> attributeCounts(), KPlayerPropertyCounts());
  connect (node, SIGNAL (attributesUpdated (const KPlayerPropertyCounts&, const KPlayerPropertyCounts&)),
    attributes(), SLOT (update (const KPlayerPropertyCounts&, const KPlayerPropertyCounts&)));
  emit layoutChanged();
}

void KPlayerListModel::vacateNode (KPlayerContainerNode* node)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListModel::vacateNode\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  disconnect (node, SIGNAL (attributesUpdated (const KPlayerPropertyCounts&, const KPlayerPropertyCounts&)),
    attributes(), SLOT (update (const KPlayerPropertyCounts&, const KPlayerPropertyCounts&)));
  attributes() -> update (KPlayerPropertyCounts(), node -> attributeCounts());
  node -> vacate();
  emit layoutChanged();
}

KPlayerNode* KPlayerListModel::nodeAt (KPlayerContainerNode* parent, int index) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerListModel::nodeAt\n";
  kdDebugTime() << " Parent " << parent -> url() << "\n";
  kdDebugTime() << " Index  " << index << "\n";
#endif
  return parent -> nodeAt (index);
}

int KPlayerListModel::nodeIndex (KPlayerContainerNode* parent, KPlayerNode* node) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerListModel::nodeIndex\n";
  kdDebugTime() << " Parent " << parent -> url() << "\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  return parent -> nodeIndex (node);
}

int KPlayerListModel::nodeCount (KPlayerContainerNode* parent) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerListModel::nodeCount\n";
  kdDebugTime() << " Parent " << parent -> url() << "\n";
#endif
  return parent -> nodeCount();
}

bool KPlayerListModel::hasNodes (KPlayerContainerNode* parent) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerListModel::hasNodes\n";
  kdDebugTime() << " Parent " << parent -> url() << "\n";
#endif
  return parent -> hasNodes();
}

int KPlayerListModel::attributeCount (void) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerListModel::attributeCount\n";
  kdDebugTime() << " Count  " << columns() << "\n";
#endif
  return columns();
}

QString KPlayerListModel::nodeData (KPlayerNode* node, int column) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerListModel::nodeData\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
  kdDebugTime() << " Column " << column << "\n";
#endif
  return column < columns() ? node -> media() -> asString (attributeName (column)) : QString::null;
}

void KPlayerListModel::setNodeData (KPlayerNode* node, int column, const QString& data) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerListModel::setNodeData\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
  kdDebugTime() << " Column " << column << "\n";
  kdDebugTime() << " Data   " << data << "\n";
#endif
  if ( column < columns() )
    node -> media() -> fromString (attributeName (column), data);
}

QString KPlayerListModel::attributeLabel (int column) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerListModel::attributeLabel\n";
  kdDebugTime() << " Column " << column << "\n";
#endif
  return column < columns() ? KPlayerMedia::info (attributeName (column)) -> caption() : QString::null;
}

void KPlayerListModel::newRootIndex (const QModelIndex& index, const QModelIndex& old)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListModel::newRootIndex\n";
#endif
  KPlayerContainerNode* container = (KPlayerContainerNode*) nodeFromIndex (index);
  attributes() -> initialize (container -> isDirectory());
  KPlayerNodeModel::newRootIndex (index, old);
}

/*void KPlayerListModel::expanded (const QModelIndex& index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListModel::expanded\n";
#endif
  KPlayerNode* node = nodeFromIndex (index);
  if ( node -> isContainer() )
  {
#ifdef DEBUG_KPLAYER_NODEVIEW
    kdDebugTime() << " Node   " << node -> url().url() << "\n";
#endif
    attributes() -> update (((KPlayerContainerNode*) node) -> attributeCounts(), KPlayerPropertyCounts());
  }
}*/

void KPlayerListModel::sort (int column, Qt::SortOrder order)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListModel::sort\n";
  kdDebugTime() << " Column " << column << "\n";
  kdDebugTime() << " Order  " << order << "\n";
#endif
  if ( column < columns() )
    KPlayerNode::setSorting (attributeName (column), order == Qt::AscendingOrder);
  KPlayerContainerNode* container = (KPlayerContainerNode*) nodeFromIndex (parent() -> rootIndex());
  container -> setCustomOrder (false);
  container -> applyOrder();
  if ( expandable() )
    for ( QModelIndex index = parent() -> topIndex(); index.isValid(); index = parent() -> indexBelow (index) )
      if ( parent() -> isExpanded (index) )
      {
        KPlayerNode* node = nodeFromIndex (index);
        if ( node -> isContainer() )
        {
          container = (KPlayerContainerNode*) node;
          container -> setCustomOrder (false);
          container -> applyOrder();
        }
      }
  emit layoutChanged();
}

void KPlayerListModel::updateColumns (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListModel::updateColumns\n";
#endif
  beginRemoveColumns (parent() -> rootIndex(), 0, parent() -> header() -> count() - 1);
  endRemoveColumns();
  beginInsertColumns (parent() -> rootIndex(), 0, columns() - 1);
  endInsertColumns();
  emit columnsUpdated();
}

QVariant KPlayerListModel::headerData (int column, Qt::Orientation orientation, int role) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerListModel::headerData\n";
#endif
  if ( orientation == Qt::Horizontal && role == Qt::SizeHintRole && column < columns() - 1 )
  {
    KConfigGroup group (config() -> group ("Multimedia Library"));
    QString key ("Column " + attributeName (column) + " Width");
    if ( group.hasKey (key) )
    {
      int size = group.readEntry (key, 200);
#ifdef DEBUG_KPLAYER_NODEMODEL
      kdDebugTime() << " Column " << column << "\n";
      kdDebugTime() << " Orientation " << orientation << "\n";
      kdDebugTime() << " Role   " << role << "\n";
      kdDebugTime() << " Size   " << size << "\n";
#endif
      return QSize (size, 0);
    }
  }
  return KPlayerNodeModel::headerData (column, orientation, role);
}

void KPlayerListModel::columnMoved (int column, int from, int to)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListModel::columnMoved\n";
  kdDebugTime() << " Column " << column << "\n";
#endif
  attributes() -> move (from, to);
}

KPlayerTreeModel::KPlayerTreeModel (QObject* parent)
  : KPlayerNodeModel (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating tree model\n";
#endif
}

KPlayerTreeModel::~KPlayerTreeModel()
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying tree model\n";
#endif
}

void KPlayerTreeModel::initialize (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerTreeModel::initialize\n";
#endif
  KPlayerNodeModel::initialize();
}

void KPlayerTreeModel::terminate (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerTreeModel::terminate\n";
#endif
  KPlayerNodeModel::terminate();
}

bool KPlayerTreeModel::populated (KPlayerContainerNode* node) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerTreeModel::populated\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  return node -> groupsPopulated() || node -> populated();
}

void KPlayerTreeModel::populateNode (KPlayerContainerNode* node)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerTreeModel::populateNode\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  node -> populateGroups();
}

void KPlayerTreeModel::vacateNode (KPlayerContainerNode* node)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerTreeModel::vacateNode\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  node -> vacateGroups();
}

KPlayerNode* KPlayerTreeModel::nodeAt (KPlayerContainerNode* parent, int index) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerTreeModel::nodeAt\n";
  kdDebugTime() << " Parent " << parent -> url() << "\n";
  kdDebugTime() << " Index  " << index << "\n";
#endif
  return parent -> containerAt (index);
}

int KPlayerTreeModel::nodeIndex (KPlayerContainerNode* parent, KPlayerNode* node) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerTreeModel::nodeIndex\n";
  kdDebugTime() << " Parent " << parent -> url() << "\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
#endif
  return parent -> containerIndex (node);
}

int KPlayerTreeModel::nodeCount (KPlayerContainerNode* parent) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerTreeModel::nodeCount\n";
  kdDebugTime() << " Parent " << parent -> url() << "\n";
#endif
  return parent -> containerCount();
}

bool KPlayerTreeModel::hasNodes (KPlayerContainerNode* parent) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerTreeModel::hasNodes\n";
  kdDebugTime() << " Parent " << parent -> url() << "\n";
#endif
  return parent -> hasContainers();
}

int KPlayerTreeModel::attributeCount (void) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerTreeModel::attributeCount\n";
#endif
  return 1;
}

QString KPlayerTreeModel::nodeData (KPlayerNode* node, int column) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerTreeModel::nodeData\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
  kdDebugTime() << " Column " << column << "\n";
#endif
  return node -> name();
}

void KPlayerTreeModel::setNodeData (KPlayerNode* node, int column, const QString& data) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerTreeModel::setNodeData\n";
  kdDebugTime() << " Node   " << node -> url() << "\n";
  kdDebugTime() << " Column " << column << "\n";
  kdDebugTime() << " Data   " << data << "\n";
#endif
  node -> media() -> setName (data);
}

QString KPlayerTreeModel::attributeLabel (int column) const
{
#ifdef DEBUG_KPLAYER_NODEMODEL
  kdDebugTime() << "KPlayerTreeModel::attributeLabel\n";
  kdDebugTime() << " Column " << column << "\n";
#endif
  return i18n("Name");
}

KPlayerNodeController::KPlayerNodeController (KActionCollection* ac, KPlayerPlaylist* playlist, KPlayerLibrary* parent)
  : QObject (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating node controller\n";
#endif
  m_ac = ac;
  m_playlist = playlist;

  m_playlists = new KPlayerContainerActionList (ki18n("%1"), ki18n("Adds selected items to %1 playlist"),
    ki18n("Add to playlist command adds the selected items to the %1 playlist."), this, "library_add_to_playlist");
  playlistActionList() -> setMaximumSize (configuration() -> playlistMenuSize());

  m_go_to = new KPlayerContainerActionList (ki18n("%1"), ki18n("Opens %1 in the library window"),
    ki18n("Go to %1 command opens the corresponding folder in the library window."), this, "library_go_to_list");

  m_columns = new KPlayerToggleActionList (listView() -> model() -> attributes() -> availableAttributes(),
    listView() -> model() -> attributes() -> attributeStates(), ki18n("Hide %1"), ki18n("Show %1"),
    ki18n("Hides %1 column"), ki18n("Shows %1 column"), ki18n("Hide %1 command hides the column."),
    ki18n("Show %1 command shows the column."), this, "library_columns");

  m_edit = new KPlayerSimpleActionList (listView() -> model() -> attributes() -> editableAttributes(), ki18n("%1"),
    ki18n("Starts edit mode for %1 field"), ki18n("Edit %1 starts edit mode for this field of the current item."),
    this, "library_edit");

  m_history_list = new KPlayerHistoryActionList (parent -> history(), ki18n("%1"), ki18n("Opens %1 in the library window"),
    ki18n("Go to %1 command opens the corresponding folder in the library window."), this, "library_history");
  connect (historyActionList(), SIGNAL (activated (int)), SLOT (goToHistory (int)));

  KAction* action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_play", action);
  action -> setText (i18n("&Play"));
  action -> setShortcut (Qt::ControlModifier + Qt::Key_Return);
  action -> setStatusTip (i18n("Plays the selected items"));
  action -> setWhatsThis (i18n("Play command plays the selected items."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_play_next", action);
  action -> setText (i18n("Play &Next"));
  action -> setStatusTip (i18n("Plays the selected items after the currently played item"));
  action -> setWhatsThis (i18n("Play next command plays the selected items after the currently played item finishes playing."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_queue", action);
  action -> setText (i18n("&Queue"));
  action -> setStatusTip (i18n("Queues the selected items"));
  action -> setWhatsThis (i18n("Queue command queues the selected items for playing."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_queue_next", action);
  action -> setText (i18n("Queue Ne&xt"));
  action -> setStatusTip (i18n("Queues the selected items for playing after the currently played item"));
  action -> setWhatsThis (i18n("Queue next command queues the selected items for playing after the currently played item finishes playing."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_add_files", action);
  action -> setText (i18n("&Files..."));
  action -> setStatusTip (i18n("Adds files to the selected folder"));
  action -> setWhatsThis (i18n("Add files command displays the standard Open File dialog and lets you choose a file or several files to add to the selected folder."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_add_url", action);
  action -> setText (i18n("&URL..."));
  action -> setStatusTip (i18n("Adds a URL to the selected folder"));
  action -> setWhatsThis (i18n("Add URL command displays the standard Open URL dialog and lets you type or paste in a URL to add to the selected folder."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_add_group", action);
  action -> setText (i18n("F&older..."));
  action -> setStatusTip (i18n("Adds a new subfolder to the selected folder"));
  action -> setWhatsThis (i18n("Add folder command allows you to add a new subfolder to the selected folder. You will need to give the new subfolder a unique name."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_add_to_new_playlist", action);
  action -> setText (i18n("&New Playlist..."));
  action -> setStatusTip (i18n("Adds selected items to a new playlist"));
  action -> setWhatsThis (i18n("Add to new playlist command prompts for a new playlist name and adds the selected items to the new playlist."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_add_to_playlists", action);
  action -> setText (i18n("&Playlists"));
  action -> setStatusTip (i18n("Adds selected items to the root playlist"));
  action -> setWhatsThis (i18n("Add to playlists command adds the selected items to the root playlist."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_add_to_collection", action);
  action -> setText (i18n("&Collection"));
  action -> setStatusTip (i18n("Adds selected items to the collection"));
  action -> setWhatsThis (i18n("Add to collection command adds the selected items to the multimedia collection."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_rename", action);
  action -> setText (i18n("&Rename"));
  action -> setShortcut (Qt::ControlModifier + Qt::Key_F2);
  action -> setStatusTip (i18n("Lets you rename the selected item"));
  action -> setWhatsThis (i18n("Rename command starts edit mode for the current item in the multimedia library so you can change the item name."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_properties", action);
  action -> setText (i18n("Propert&ies..."));
  action -> setStatusTip (i18n("Opens Properties dialog for the selected item"));
  action -> setWhatsThis (i18n("Properties command opens File Properties dialog for the current item in the multimedia library. See the File properties micro-HOWTO for details."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_select_all", action);
  action -> setText (i18n("&Select All"));
  action -> setStatusTip (i18n("Selects all items in the current folder"));
  action -> setWhatsThis (i18n("Select all command selects all items in the current folder of the multimedia library."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_move_up", action);
  action -> setText (i18n("Move &Up"));
  action -> setIcon (KIcon ("arrow-up"));
  action -> setStatusTip (i18n("Moves the selected items up in the playlist"));
  action -> setWhatsThis (i18n("Move Up command moves the selected items up in a playlist. You can also move items around by clicking and dragging them with the left mouse button."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_move_down", action);
  action -> setText (i18n("Move &Down"));
  action -> setIcon (KIcon ("arrow-down"));
  action -> setStatusTip (i18n("Moves the selected items down in the playlist"));
  action -> setWhatsThis (i18n("Move Down command moves the selected items down in a playlist. You can also move items around by clicking and dragging them with the left mouse button."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_remove", action);
  action -> setText (i18n("Re&move"));
  action -> setIcon (KIcon ("list-remove"));
  action -> setShortcut (Qt::Key_Delete);
  action -> setStatusTip (i18n("Removes the selected items from the multimedia library"));
  action -> setWhatsThis (i18n("Remove command removes the selected items from the multimedia library."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_go_back", action);
  action -> setText (i18n("&Back"));
  action -> setIcon (KIcon ("go-previous"));
  action -> setShortcut (Qt::AltModifier + Qt::Key_Left);
  action -> setStatusTip (i18n("Opens the previous folder"));
  action -> setWhatsThis (i18n("Go back command opens the previous folder from the history."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_go_forward", action);
  action -> setText (i18n("&Forward"));
  action -> setIcon (KIcon ("go-next"));
  action -> setShortcut (Qt::AltModifier + Qt::Key_Right);
  action -> setStatusTip (i18n("Opens the next folder"));
  action -> setWhatsThis (i18n("Go forward command opens the next folder from the history."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_go_up", action);
  action -> setText (i18n("&Up"));
  action -> setIcon (KIcon ("go-up"));
  action -> setShortcut (Qt::AltModifier + Qt::ShiftModifier + Qt::Key_Up);
  action -> setStatusTip (i18n("Opens the parent folder"));
  action -> setWhatsThis (i18n("Go up command opens the parent folder of the current folder."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_go_down", action);
  action -> setText (i18n("&Down"));
  action -> setIcon (KIcon ("go-down"));
  action -> setShortcut (Qt::AltModifier + Qt::ShiftModifier + Qt::Key_Down);
  action -> setStatusTip (i18n("Opens the selected folder"));
  action -> setWhatsThis (i18n("Go down command opens the selected folder."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("library_go_to_origin", action);
  action -> setText (i18n("&To Origin"));
  action -> setShortcut (Qt::AltModifier + Qt::ShiftModifier + Qt::Key_Left);
  action -> setStatusTip (i18n("Opens the origin of the current folder"));
  action -> setWhatsThis (i18n("Go to origin command opens the origin folder of the current folder."));

  action = new KAction (actionCollection());
  actionCollection() -> addAction ("playlist_edit", action);
  connect (action, SIGNAL (triggered()), SLOT (editPlaylist()));
  action -> setText (i18n("&Edit"));
  action -> setStatusTip (i18n("Opens the playlist in the multimedia library"));
  action -> setWhatsThis (i18n("Edit command opens the playlist in the multimedia library and allows you to edit it."));
}

KPlayerNodeController::~KPlayerNodeController()
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying node controller\n";
#endif
}

void KPlayerNodeController::initialize (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::initialize\n";
#endif
  playlistActionList() -> initialize (KUrl ("kplayer:/playlists"));
  goToActionList() -> initialize (KUrl ("kplayer:/"));
  disconnectActions();
}

void KPlayerNodeController::terminate (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::terminate\n";
#endif
  playlistActionList() -> terminate();
  goToActionList() -> terminate();
}

void KPlayerNodeController::disconnectActions (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Disconnecting actions\n";
#endif
  QAction* a = action ("library_play");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(play()));
  a -> setEnabled (false);
  a = action ("library_play_next");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(playNext()));
  a -> setEnabled (false);
  a = action ("library_queue");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(queue()));
  a -> setEnabled (false);
  a = action ("library_queue_next");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(queueNext()));
  a -> setEnabled (false);
  a = action ("library_add_files");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(addFiles()));
  a -> setEnabled (false);
  a = action ("library_add_url");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(addUrl()));
  a -> setEnabled (false);
  a = action ("library_add_group");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(addGroup()));
  a -> setEnabled (false);
  a = action ("library_add_to_new_playlist");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(addToNewPlaylist()));
  a -> setEnabled (false);
  a = action ("library_add_to_playlists");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(addToPlaylists()));
  a -> setEnabled (false);
  playlistActionList() -> disconnect (SIGNAL(activated(KPlayerNode*)), this, SLOT(addToPlaylist(KPlayerNode*)));
  a = action ("library_add_to_collection");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(addToCollection()));
  a -> setEnabled (false);
  a = action ("library_rename");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(editName()));
  a -> setEnabled (false);
  editActionList() -> disconnect (SIGNAL(activated(int)), this, SLOT(editField(int)));
  a = action ("library_properties");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(properties()));
  a -> setEnabled (false);
  a = action ("library_select_all");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(selectAll()));
  a -> setEnabled (false);
  a = action ("library_move_up");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(moveUp()));
  a -> setEnabled (false);
  a = action ("library_move_down");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(moveDown()));
  a -> setEnabled (false);
  a = action ("library_remove");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(remove()));
  a -> setEnabled (false);
  columnActionList() -> disconnect (SIGNAL(activated(int)), this, SLOT(toggleColumn(int)));
  a = action ("library_go_back");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(goBack()));
  a -> setEnabled (false);
  a = action ("library_go_forward");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(goForward()));
  a -> setEnabled (false);
  a = action ("library_go_up");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(goUp()));
  a -> setEnabled (false);
  a = action ("library_go_down");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(goDown()));
  a -> setEnabled (false);
  a = action ("library_go_to_origin");
  a -> disconnect (SIGNAL(triggered()), this, SLOT(goToOrigin()));
  a -> setEnabled (false);
  emit enableActionGroup ("library_add", false);
  emit enableActionGroup ("library_add_to", false);
  emit enableActionGroup ("library_columns", false);
  emit enableActionGroup ("library_edit", false);
}

void KPlayerNodeController::connectActions (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Connecting actions\n";
#endif
  disconnectActions();
  connect (action ("library_play"), SIGNAL(triggered()), SLOT(play()));
  connect (action ("library_play_next"), SIGNAL(triggered()), SLOT(playNext()));
  connect (action ("library_queue"), SIGNAL(triggered()), SLOT(queue()));
  connect (action ("library_queue_next"), SIGNAL(triggered()), SLOT(queueNext()));
  connect (action ("library_add_files"), SIGNAL(triggered()), SLOT(addFiles()));
  connect (action ("library_add_url"), SIGNAL(triggered()), SLOT(addUrl()));
  connect (action ("library_add_group"), SIGNAL(triggered()), SLOT(addGroup()));
  connect (action ("library_add_to_new_playlist"), SIGNAL(triggered()), SLOT(addToNewPlaylist()));
  connect (action ("library_add_to_playlists"), SIGNAL(triggered()), SLOT(addToPlaylists()));
  connect (playlistActionList(), SIGNAL(activated(KPlayerNode*)), SLOT(addToPlaylist(KPlayerNode*)));
  connect (action ("library_add_to_collection"), SIGNAL(triggered()), SLOT(addToCollection()));
  connect (action ("library_rename"), SIGNAL(triggered()), SLOT(editName()));
  connect (editActionList(), SIGNAL(activated(int)), SLOT(editField(int)));
  connect (action ("library_properties"), SIGNAL(triggered()), SLOT(properties()));
  connect (action ("library_select_all"), SIGNAL(triggered()), SLOT(selectAll()));
  connect (action ("library_move_up"), SIGNAL(triggered()), SLOT(moveUp()));
  connect (action ("library_move_down"), SIGNAL(triggered()), SLOT(moveDown()));
  connect (action ("library_remove"), SIGNAL(triggered()), SLOT(remove()));
  connect (columnActionList(), SIGNAL(activated(int)), SLOT(toggleColumn(int)));
  connect (action ("library_go_back"), SIGNAL(triggered()), SLOT(goBack()));
  connect (action ("library_go_forward"), SIGNAL(triggered()), SLOT(goForward()));
  connect (action ("library_go_up"), SIGNAL(triggered()), SLOT(goUp()));
  connect (action ("library_go_down"), SIGNAL(triggered()), SLOT(goDown()));
  connect (action ("library_go_to_origin"), SIGNAL(triggered()), SLOT(goToOrigin()));
  updateActions();
}

void KPlayerNodeController::updateActions (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Updating actions\n";
#endif
  KPlayerNodeView* view = parent() -> activeView();
  KPlayerNodeModel* model = view -> model();
  bool selection = view -> selectionModel() -> hasSelection();
  KPlayerNode* node = model -> nodeFromIndex (view -> activeIndex());
  KPlayerContainerNode* container = (KPlayerContainerNode*) model -> nodeFromIndex (view -> activeContainer());
  KPlayerContainerNode* root = (KPlayerContainerNode*) model -> nodeFromIndex (view -> rootIndex());
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Selection " << selection << "\n";
  if ( node )
    kdDebugTime() << " Active " << node -> url().url() << "\n";
  if ( container )
    kdDebugTime() << " Container " << container -> url().url() << "\n";
#endif
  action ("library_play") -> setEnabled (selection);
  action ("library_play_next") -> setEnabled (selection);
  bool enable = selection && container -> canQueue();
  action ("library_queue") -> setEnabled (enable);
  action ("library_queue_next") -> setEnabled (enable);
  enable = container && (container -> canAddLeaves() || container -> canAddBranches());
  emit enableActionGroup ("library_add", enable);
  enable = container && container -> canAddLeaves();
  action ("library_add_files") -> setEnabled (enable);
  action ("library_add_url") -> setEnabled (enable);
  enable = container && container -> canAddBranches();
  QAction* a = action ("library_add_group");
  a -> setEnabled (enable);
  if ( container && container -> isDevices() )
  {
    a -> setText (i18n("&Device..."));
    a -> setStatusTip (i18n("Adds a new device"));
    a -> setWhatsThis (i18n("Add device command allows you to add a new device. You will need to give the new device a unique name and specify the device path and type."));
  }
  else if ( container && container -> isPlaylist() )
  {
    a -> setText (i18n("&Playlist..."));
    a -> setStatusTip (i18n("Adds a new playlist"));
    a -> setWhatsThis (i18n("Add playlist command allows you to add a new playlist. You will need to give the new playlist a unique name."));
  }
  else
  {
    a -> setText (i18n("F&older..."));
    a -> setStatusTip (i18n("Adds a new subfolder to the selected folder"));
    a -> setWhatsThis (i18n("Add folder command allows you to add a new subfolder to the selected folder. You will need to give the new subfolder a unique name."));
  }
  enable = selection && container -> canSaveAsPlaylist();
  emit enableActionGroup ("library_add_to", enable);
  action ("library_add_to_new_playlist") -> setEnabled (enable);
  action ("library_add_to_playlists") -> setEnabled (enable);
  action ("library_add_to_collection") -> setEnabled (enable);
  action ("library_rename") -> setEnabled (node && node -> canRename());
  emit enableActionGroup ("library_edit", view == listView() && node && ! node -> isContainer());
  action ("library_properties") -> setEnabled (node && (node -> hasProperties() || node -> isContainer()
    && ((KPlayerContainerNode*) node) -> origin() && ((KPlayerContainerNode*) node) -> origin() -> hasProperties()));
  enable = selection && container -> allowsCustomOrder();
  action ("library_move_up") -> setEnabled (enable);
  action ("library_move_down") -> setEnabled (enable);
  action ("library_remove") -> setEnabled (selection && container -> canRemove());
  emit enableActionGroup ("library_columns", ! columnActionList() -> isEmpty());
  action ("library_go_down") -> setEnabled (container != root);
  action ("library_go_to_origin") -> setEnabled (container -> origin() != 0);
  action ("library_go_back") -> setEnabled (parent() -> history() -> canGoBack());
  action ("library_go_forward") -> setEnabled (parent() -> history() -> canGoForward());
  container = (KPlayerContainerNode*) model -> nodeFromIndex (listView() -> rootIndex());
  action ("library_go_up") -> setEnabled (container -> parent() && container -> parent() -> parent());
  container = (KPlayerContainerNode*) model -> nodeFromIndex (treeView() -> activeIndex());
  action ("library_select_all") -> setEnabled (container -> hasNodes());
}

KPlayerNodeList KPlayerNodeController::selection (void) const
{
  KPlayerNodeView* view = parent() -> activeView();
  return view -> model() -> nodesFromIndexList (view -> selection());
}

void KPlayerNodeController::play (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::play\n";
#endif
  playlist() -> play (selection());
}

void KPlayerNodeController::playNext (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::playNext\n";
#endif
  playlist() -> playNext (selection());
}

void KPlayerNodeController::queue (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::queue\n";
#endif
  playlist() -> queue (selection());
}

void KPlayerNodeController::queueNext (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::queueNext\n";
#endif
  playlist() -> queueNext (selection());
}

void KPlayerNodeController::addFiles (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::addFiles\n";
#endif
  activeContainer() -> append (kPlayerEngine() -> openFiles (i18n("Add files")));
}

void KPlayerNodeController::addUrl (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::addUrl\n";
#endif
  activeContainer() -> append (kPlayerEngine() -> openUrl (i18n("Add URL")));
}

void KPlayerNodeController::addGroup (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::addGroup\n";
#endif
  KPlayerContainerNode* container = activeContainer();
  if ( container -> isDevices() )
  {
    KPlayerDeviceDialog dialog (parent());
    if ( dialog.exec() == KDialog::Accepted )
      dialog.addDevice();
  }
  else
  {
    KPlayerNodeNameValidator validator (container);
    QString name = KInputDialog::getText (container -> isPlaylist() ? i18n("Add playlist") : i18n("Add folder"),
      container -> isPlaylist() ? i18n("Playlist name") : i18n("Folder name"),
      QString::null, 0, 0, &validator, QString::null,
      i18n("Folder name field allows you to enter a name for a new folder. OK button will be enabled when you enter a unique and valid name."));
    if ( ! name.isNull() )
      container -> addBranch (name);
  }
}

void KPlayerNodeController::addToNewPlaylist (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::addToNewPlaylist\n";
#endif
  addToNewPlaylist (selection());
}

void KPlayerNodeController::addToNewPlaylistUrls (const KUrl::List& urls)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::addToNewPlaylist urls\n";
  kdDebugTime() << " URLs   " << urls.count() << "\n";
#endif
  KPlayerContainerNode* container = playlistActionList() -> node();
  if ( playlist() -> checkUrls (urls) )
  {
    KPlayerNodeList list (KPlayerNodeList::fromUrlList (urls));
    container = addToNewPlaylist (list);
    list.releaseAll();
  }
  if ( container )
    parent() -> open (container);
}

KPlayerContainerNode* KPlayerNodeController::addToNewPlaylist (const KPlayerNodeList& list)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::addToNewPlaylist list\n";
  kdDebugTime() << " Count  " << list.count() << "\n";
#endif
  if ( list.isEmpty() )
    return 0;
  KPlayerContainerNode* container = playlistActionList() -> node();
  KPlayerNodeNameValidator validator (container);
  QString name = KInputDialog::getText (i18n("Add to new playlist"), i18n("Playlist name"),
    QString::null, 0, 0, &validator, QString::null,
    i18n("Playlist name field allows you to enter a name for a new playlist. OK button will be enabled when you enter a unique and valid name."));
  if ( ! name.isNull() )
  {
    container -> addBranch (name);
    container = container -> getNodeById (name);
    if ( container )
    {
      KPlayerContainerNode* node = list.count() == 1
        && list.first() -> isContainer() ? (KPlayerContainerNode*) list.first() : 0;
      if ( node )
        node -> populate();
      const KPlayerNodeList& nodes = node ? node -> nodes() : list;
      if ( ! nodes.isEmpty() )
        container -> append (nodes);
      if ( node )
        node -> vacate();
    }
    return container;
  }
  return 0;
}

void KPlayerNodeController::addToPlaylistsUrls (const KUrl::List& urls)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::addToPlaylists\n";
  kdDebugTime() << " URLs   " << urls.count() << "\n";
#endif
  if ( playlist() -> checkUrls (urls) )
  {
    KPlayerNodeList list (KPlayerNodeList::fromUrlList (urls));
    if ( ! list.isEmpty() )
    {
      playlistActionList() -> node() -> append (list);
      list.releaseAll();
    }
  }
  parent() -> open (playlistActionList() -> node());
}

void KPlayerNodeController::addToPlaylists (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::addToPlaylists\n";
#endif
  KPlayerNodeList list (selection());
  if ( ! list.isEmpty() )
    if ( list.count() == 1 && list.first() == playlist() -> nowplaying() )
      playlist() -> addToPlaylists();
    else
      playlistActionList() -> node() -> append (list);
}

void KPlayerNodeController::addToPlaylist (KPlayerNode* node)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::addToPlaylist\n";
  kdDebugTime() << " Node   " << node -> url().url() << "\n";
#endif
  KPlayerNodeList list (selection());
  KPlayerContainerNode* container = list.count() == 1
    && list.first() -> isContainer() ? (KPlayerContainerNode*) list.first() : 0;
  if ( container )
    container -> populate();
  const KPlayerNodeList& nodes = container ? container -> nodes() : list;
  if ( ! nodes.isEmpty() && node -> isContainer() )
    ((KPlayerContainerNode*) node) -> append (nodes);
  if ( container )
    container -> vacate();
}

void KPlayerNodeController::addToCollectionUrls (const KUrl::List& urls)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::addToCollection\n";
  kdDebugTime() << " URLs   " << urls.count() << "\n";
#endif
  KPlayerContainerNode* node = KPlayerNode::root() -> getNodeByUrl (KUrl ("kplayer:/collection"));
  if ( playlist() -> checkUrls (urls) )
  {
    KPlayerNodeList list (KPlayerNodeList::fromUrlList (urls));
    if ( ! list.isEmpty() )
    {
      node -> add (list);
      list.releaseAll();
    }
  }
  parent() -> open (node);
}

void KPlayerNodeController::addToCollection (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::addToCollection\n";
#endif
  KPlayerNodeList list (selection());
  if ( ! list.isEmpty() )
    if ( list.count() == 1 && list.first() == playlist() -> nowplaying() )
      playlist() -> addToCollection();
    else
      KPlayerNode::root() -> getNodeByUrl (KUrl ("kplayer:/collection")) -> add (list);
}

void KPlayerNodeController::editName (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::editName\n";
#endif
  parent() -> activeView() -> edit (parent() -> activeView() -> activeIndex());
}

void KPlayerNodeController::properties (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::properties\n";
#endif
  KPlayerNodeView* view = parent() -> activeView();
  KPlayerNode* node = view -> model() -> nodeFromIndex (view -> activeIndex());
  if ( node && ! node -> hasProperties() && node -> isContainer() )
    node = ((KPlayerContainerNode*) node) -> origin();
  if ( node && node -> hasProperties() )
  {
    KPlayerPropertiesDialog* dialog;
    if ( ! node -> isContainer() )
      dialog = KPlayerPropertiesDialog::createDialog ((KPlayerTrackProperties*) node -> media());
    else
    {
      const QString& type = ((KPlayerDeviceProperties*) node -> media()) -> type();
      if ( type == "TV" )
        dialog = new KPlayerTVDevicePropertiesDialog;
      else if ( type == "DVB" )
        dialog = new KPlayerDVBDevicePropertiesDialog;
      else
        dialog = new KPlayerDevicePropertiesDialog;
    }
    dialog -> setup (node -> media() -> url());
    dialog -> exec();
    delete dialog;
  }
}

void KPlayerNodeController::moveUp (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::moveUp\n";
#endif
  KPlayerNodeView* view = parent() -> activeView();
  KPlayerNodeModel* model = view -> model();
  QModelIndex parent = view -> rootIndex();
  QModelIndex index = view -> topIndex();
  do
  {
    if ( model -> parent (index) == parent )
    {
      QModelIndexList list = view -> contiguousSelection (index, false);
      QModelIndex last = parent;
      while ( ! list.isEmpty() )
      {
        QModelIndex after = list.count() == 1 ? last : list.at (list.count() - 2);
        last = list.last();
        list = view -> contiguousSelection (last, true);
        if ( ! list.isEmpty() )
        {
          model -> move (list, after);
          list = view -> contiguousSelection (list.last(), false);
        }
      }
    }
    parent = index;
    index = view -> indexBelow (index);
  }
  while ( index.isValid() );
}

void KPlayerNodeController::moveDown (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::moveDown\n";
#endif
  KPlayerNodeView* view = parent() -> activeView();
  KPlayerNodeModel* model = view -> model();
  QModelIndex parent = view -> rootIndex();
  QModelIndex index = view -> topIndex();
  do
  {
    if ( model -> parent (index) == parent )
    {
      QModelIndexList selected = view -> contiguousSelection (index, true);
      while ( ! selected.isEmpty() )
      {
        QModelIndexList unselected = view -> contiguousSelection (selected.last(), false);
        if ( ! unselected.isEmpty() )
        {
          model -> move (selected, unselected.first());
          selected = view -> contiguousSelection (unselected.last(), true);
        }
      }
    }
    parent = index;
    index = view -> indexBelow (index);
  }
  while ( index.isValid() );
}

void KPlayerNodeController::remove (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::remove\n";
#endif
  KPlayerNodeView* view = parent() -> activeView();
  KPlayerNodeModel* model = view -> model();
  QModelIndex parent = view -> rootIndex();
  QModelIndex index = view -> topIndex();
  do
  {
    if ( model -> parent (index) == parent )
      model -> remove (view -> selection (index));
    parent = index;
    index = view -> indexBelow (index);
  }
  while ( index.isValid() );
}

void KPlayerNodeController::selectAll (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::selectAll\n";
#endif
  parent() -> listView() -> selectAll();
}

void KPlayerNodeController::goBack (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::goBack\n";
#endif
  parent() -> history() -> goBack();
}

void KPlayerNodeController::goForward (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::goForward\n";
#endif
  parent() -> history() -> goForward();
}

void KPlayerNodeController::goToHistory (int index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::goToHistory\n";
  kdDebugTime() << " Index  " << index << "\n";
#endif
  parent() -> history() -> goTo (index);
}

void KPlayerNodeController::goUp (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::goUp\n";
#endif
  parent() -> open (listView() -> model() -> nodeFromIndex (listView() -> rootIndex()) -> parent());
}

void KPlayerNodeController::goDown (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::goDown\n";
#endif
  parent() -> open (activeContainer());
}

void KPlayerNodeController::goToOrigin (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::goToOrigin\n";
#endif
  KPlayerContainerNode* container = activeContainer();
  if ( container && container -> origin() )
    parent() -> open (container -> origin());
}

void KPlayerNodeController::toggleColumn (int index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::toggleColumn\n";
  kdDebugTime() << " Index  " << index << "\n";
#endif
  const QString& name (listView() -> model() -> attributes() -> availableAttributes() [index]);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Name " << name << "\n";
#endif
  listView() -> model() -> attributes() -> toggleState (name);
}

void KPlayerNodeController::editField (int index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::editField\n";
  kdDebugTime() << " Index  " << index << "\n";
#endif
  index = listView() -> model() -> attributes() -> editableIndex (index);
  if ( index >= 0 )
  {
#ifdef DEBUG_KPLAYER_NODEVIEW
    kdDebugTime() << " Column " << index << "\n";
#endif
    parent() -> activeView() -> edit (parent() -> activeView() -> activeIndex());
  }
}

void KPlayerNodeController::editPlaylist (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeController::editPlaylist\n";
#endif
  parent() -> open (playlist() -> nowplaying());
}

KPlayerNodeView::KPlayerNodeView (QWidget* parent)
  : QTreeView (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating node view\n";
#endif
  setAnimated (false);
  setContextMenuPolicy (Qt::CustomContextMenu);
  setEditTriggers (EditKeyPressed);
  setUniformRowHeights (true);
}

KPlayerNodeView::~KPlayerNodeView()
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying node view\n";
#endif
}

void KPlayerNodeView::initialize (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Initializing node view\n";
#endif
  model() -> initialize();
  connect (this, SIGNAL (expanded (const QModelIndex&)), SLOT (populateIndex (const QModelIndex&)));
  connect (this, SIGNAL (collapsed (const QModelIndex&)), SLOT (vacateIndex (const QModelIndex&)));
}

void KPlayerNodeView::terminate (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Terminating node view\n";
#endif
  model() -> terminate();
  setModel (0);
}

void KPlayerNodeView::setRootIndex (const QModelIndex& index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeView::setRootIndex\n";
#endif
  static bool recursion = false;
  if ( recursion )
    return;
  recursion = true;
  setCurrentIndex (QModelIndex());
  if ( model() )
  {
    model() -> newRootIndex (index, rootIndex());
    setItemsExpandable (model() -> expandable());
    setRootIsDecorated (model() -> expandable());
  }
  reset();
  QTreeView::setRootIndex (index);
  recursion = false;
  /*if ( model() && model() -> expandable() )
  {
    bool emitLayoutChanged = false;
    for ( QModelIndex idx = indexBelow (index); model() -> ancestor (idx, index); idx = indexBelow (idx) )
      if ( isExpanded (idx) )
      {
        model() -> expanded (idx);
        emitLayoutChanged = true;
      }
    if ( emitLayoutChanged )
      emit layoutChanged();
  }*/
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Columns " << header() -> count() << "\n";
#endif
}

void KPlayerNodeView::populateIndex (const QModelIndex& index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeView::populateIndex\n";
#endif
  if ( model() -> ancestor (index, rootIndex()) )
  {
    model() -> populateIndex (index);
    for ( QModelIndex idx = indexBelow (index); model() -> ancestor (idx, index); idx = indexBelow (idx) )
      if ( isExpanded (idx) )
        model() -> populateIndex (idx);
  }
}

void KPlayerNodeView::vacateIndex (const QModelIndex& index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeView::vacateIndex\n";
#endif
  if ( model() -> ancestor (index, rootIndex()) )
  {
    if ( model() -> ancestor (currentIndex(), index) )
      setCurrentIndex (index);
    model() -> vacateIndex (index);
  }
}

QModelIndex KPlayerNodeView::lastVisibleDescendant (const QModelIndex& parent) const
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeView::lastVisibleDescendant\n";
#endif
  QModelIndex index = parent;
  for ( QModelIndex idx = indexBelow (index); model() -> ancestor (idx, parent); idx = indexBelow (index) )
    index = idx;
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Node   " << model() -> nodeFromIndex (index) -> url() << "\n";
#endif
  return index;
}

QModelIndexList KPlayerNodeView::selection (void) const
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeView::selection\n";
#endif
  QModelIndexList list;
  for ( QModelIndex index = topIndex(); index.isValid(); index = indexBelow (index) )
    if ( selectionModel() -> isSelected (index) )
    {
      list.append (index);
      index = lastVisibleDescendant (index);
    }
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Count  " << list.count() << "\n";
#endif
  return list;
}

QModelIndexList KPlayerNodeView::selection (const QModelIndex& start) const
{
  QModelIndex index = start;
  QModelIndex parent = model() -> parent (index);
  QModelIndexList list;
  while ( model() -> ancestor (index, parent) )
  {
    if ( model() -> parent (index) == parent && selectionModel() -> isSelected (index) )
      list.append (index);
    index = indexBelow (index);
  }
  return list;
}

QModelIndexList KPlayerNodeView::contiguousSelection (const QModelIndex& start, bool selected) const
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeView::contiguousSelection\n";
#endif
  QModelIndex index = start;
  QModelIndex parent = model() -> parent (index);
  QModelIndexList list;
  while ( model() -> ancestor (index, parent) )
  {
    if ( model() -> parent (index) == parent )
      if ( selectionModel() -> isSelected (index) == selected )
        list.append (index);
      else if ( ! list.isEmpty() )
        break;
    index = indexBelow (index);
  }
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Count  " << list.count() << "\n";
#endif
  return list;
}

void KPlayerNodeView::focusInEvent (QFocusEvent* event)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Node view getting focus\n";
#endif
  QTreeView::focusInEvent (event);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Node view got focus\n";
#endif
  emit activated (this);
}

void KPlayerNodeView::focusOutEvent (QFocusEvent* event)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Node view loosing focus\n";
#endif
  QTreeView::focusOutEvent (event);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Node view lost focus\n";
#endif
}

KPlayerListView::KPlayerListView (QWidget* parent)
  : KPlayerNodeView (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating list view\n";
#endif
  setModel (new KPlayerListModel (this));
}

KPlayerListView::~KPlayerListView()
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying list view\n";
#endif
}

void KPlayerListView::initialize (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Initializing list view\n";
#endif
  setSelectionMode (ExtendedSelection);
  setSortingEnabled (true);
  KPlayerNodeView::initialize();
  connect (model(), SIGNAL (columnsUpdated()), SLOT (updateColumns()));
  connect (header(), SIGNAL (sectionResized (int, int, int)), SLOT (columnResized (int, int, int)));
  connect (header(), SIGNAL (sectionMoved (int, int, int)), model(), SLOT (columnMoved (int, int, int)));
}

void KPlayerListView::terminate (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Terminating list view\n";
#endif
  KPlayerNodeView::terminate();
}

void KPlayerListView::setRootIndex (const QModelIndex& index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListView::setRootIndex\n";
#endif
  static bool recursion = false;
  if ( recursion )
    return;
  recursion = true;
  KPlayerNodeView::setRootIndex (index);
  if ( model() )
    updateColumns();
  recursion = false;
}

void KPlayerListView::updateColumns (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListView::updateColumns\n";
#endif
  disconnect (header(), SIGNAL (sectionResized (int, int, int)), this, SLOT (columnResized (int, int, int)));
  KConfigGroup group (config() -> group ("Multimedia Library"));
  int beforeLast = model() -> columns() - 1;
  for ( int column = 0; column < beforeLast; column ++ )
  {
    int size = group.readEntry ("Column " + model() -> attributeName (column) + " Width", 0);
    if ( size > 0 )
    {
#ifdef DEBUG_KPLAYER_NODEMODEL
      kdDebugTime() << " Column " << column << "\n";
      kdDebugTime() << " Size   " << size << "\n";
#endif
      header() -> resizeSection (column, size);
    }
    else
      resizeColumnToContents (column);
  }
  connect (header(), SIGNAL (sectionResized (int, int, int)), this, SLOT (columnResized (int, int, int)));
}

void KPlayerListView::columnResized (int column, int old, int size)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerListView::columnResized\n";
  kdDebugTime() << " Column " << column << "\n";
  kdDebugTime() << " Old size " << old << "\n";
  kdDebugTime() << " New size " << size << "\n";
#endif
  if ( size > 0 && column < model() -> columns() - 1 )
    config() -> group ("Multimedia Library").writeEntry ("Column " + model() -> attributeName (column) + " Width", size);
}

KPlayerTreeView::KPlayerTreeView (QWidget* parent)
  : KPlayerNodeView (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating tree view\n";
#endif
  setModel (new KPlayerTreeModel (this));
}

KPlayerTreeView::~KPlayerTreeView()
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying tree view\n";
#endif
}

void KPlayerTreeView::initialize (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Initializing tree view\n";
#endif
  setSelectionMode (SingleSelection);
  KPlayerNodeView::initialize();
  setRootIndex (QModelIndex());
  setCurrentIndex (topIndex());
}

void KPlayerTreeView::terminate (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Terminating tree view\n";
#endif
  KPlayerNodeView::terminate();
}

void KPlayerTreeView::setActiveIndex (const QModelIndex& index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerTreeView::setActiveIndex\n";
#endif
  if ( index.isValid() && index != currentIndex() )
  {
    scrollTo (lastVisibleDescendant (index));
    scrollTo (index);
    setCurrentIndex (index);
  }
}

KPlayerLibrary::KPlayerLibrary (KActionCollection* ac, KPlayerPlaylist* playlist, QWidget* parent)
  : QSplitter (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating library\n";
#endif
  m_popup_menu_shown = false;
  m_history = new KPlayerHistory (this);
  m_tree = new KPlayerTreeView (this);
  //setResizeMode (treeView(), QSplitter::KeepSize);
  m_list = new KPlayerListView (this);
  m_controller = new KPlayerNodeController (ac, playlist, this);
  connect (controller() -> goToActionList(), SIGNAL (activated (KPlayerNode*)), SLOT (open (KPlayerNode*)));
  connect (history(), SIGNAL (updated()), controller() -> historyActionList(), SLOT (update()));
  connect (listView(), SIGNAL (doubleClicked (const QModelIndex&)), SLOT (listIndexActivated (const QModelIndex&)));
  connect (listView(), SIGNAL (activated (const QModelIndex&)), SLOT (listIndexActivated (const QModelIndex&)));
  connect (listView() -> model() -> attributes(), SIGNAL (updated()), controller() -> columnActionList(), SLOT (update()));
  connect (listView() -> model() -> attributes(), SIGNAL (updated()), controller() -> editActionList(), SLOT (update()));

  QList<int> sizes;
  sizes.append (config() -> group ("Multimedia Library").readEntry ("Tree View Width", 150));
  sizes.append (config() -> group ("Multimedia Library").readEntry ("List View Width", 350));
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Tree width " << sizes.first() << "\n";
  kdDebugTime() << " List width " << sizes.last() << "\n";
#endif
  setSizes (sizes);

  m_active_view = m_tree;
  m_height = config() -> group ("General Options").readEntry ("Playlist Height", 300);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Height " << m_height << "\n";
#endif
}

KPlayerLibrary::~KPlayerLibrary()
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Destroying library\n";
#endif
}

void KPlayerLibrary::initialize (QMenu* menu)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerLibrary::initialize\n";
#endif
  m_popup = menu;
  setActiveView (treeView());
  connect (parent(), SIGNAL (visibilityChanged (bool)), SLOT (visibilityChanged (bool)));
  connect (menu, SIGNAL (aboutToHide()), SLOT (contextMenuHidden()));
  connect (listView() -> model(), SIGNAL (layoutChanged()), controller(), SLOT (updateActions()));
  connect (treeView() -> model(), SIGNAL (layoutChanged()), controller(), SLOT (updateActions()));
  connect (listView() -> model() -> attributes(), SIGNAL (updated()), controller(), SLOT (updateActions()));
  connect (listView() -> selectionModel(), SIGNAL (selectionChanged (const QItemSelection&, const QItemSelection&)),
    SLOT (selectionChanged (const QItemSelection&, const QItemSelection&)));
  connect (treeView() -> selectionModel(), SIGNAL (selectionChanged (const QItemSelection&, const QItemSelection&)),
    SLOT (selectionChanged (const QItemSelection&, const QItemSelection&)));
  connect (treeView() -> selectionModel(), SIGNAL (currentChanged (const QModelIndex&, const QModelIndex&)),
    SLOT (currentIndexChanged (const QModelIndex&, const QModelIndex&)));
  connect (listView(), SIGNAL (customContextMenuRequested (const QPoint&)), SLOT (showContextMenu (const QPoint&)));
  connect (treeView(), SIGNAL (customContextMenuRequested (const QPoint&)), SLOT (showContextMenu (const QPoint&)));
  connect (listView(), SIGNAL (activated (KPlayerNodeView*)), SLOT (setActiveView (KPlayerNodeView*)));
  connect (treeView(), SIGNAL (activated (KPlayerNodeView*)), SLOT (setActiveView (KPlayerNodeView*)));
  controller() -> initialize();
  listView() -> initialize();
  treeView() -> initialize();
}

void KPlayerLibrary::terminate (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerLibrary::terminate\n";
  kdDebugTime() << " Tree width " << sizes().first() << "\n";
  kdDebugTime() << " List width " << sizes().last() << "\n";
#endif
  disconnect (listView(), SIGNAL (activated (KPlayerNodeView*)), this, SLOT (setActiveView (KPlayerNodeView*)));
  disconnect (treeView(), SIGNAL (activated (KPlayerNodeView*)), this, SLOT (setActiveView (KPlayerNodeView*)));
  config() -> group ("Multimedia Library").writeEntry ("Tree View Width", sizes().first());
  config() -> group ("Multimedia Library").writeEntry ("List View Width", sizes().last());
  config() -> group ("General Options").writeEntry ("Playlist Height", m_height);
  listView() -> terminate();
  treeView() -> terminate();
  controller() -> terminate();
  KPlayerNode::terminate();
}

void KPlayerLibrary::setActiveView (KPlayerNodeView* view)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerLibrary::setActiveView\n";
#endif
  m_active_view = view;
  controller() -> updateActions();
}

void KPlayerLibrary::showContextMenu (const QPoint& position)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerLibrary::showContextMenu\n";
  kdDebugTime() << " Local  " << position.x() << "x" << position.y() << "\n";
#endif
  QPoint pos = activeView() -> mapToGlobal (position);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Global " << pos.x() << "x" << pos.y() << "\n";
#endif
  m_popup_menu_shown = true;
  m_popup -> popup (pos);
}

void KPlayerLibrary::contextMenuHidden (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerLibrary::contextMenuHidden\n";
#endif
  m_popup_menu_shown = false;
  QTimer::singleShot (0, this, SLOT (resetActiveIndex()));
}

void KPlayerLibrary::visibilityChanged (bool visible)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerLibrary::visibilityChanged\n";
#endif
  if ( visible )
    controller() -> connectActions();
  else
    controller() -> disconnectActions();
}

void KPlayerLibrary::selectionChanged (const QItemSelection&, const QItemSelection&)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerLibrary::selectionChanged\n";
#endif
  controller() -> updateActions();
}

void KPlayerLibrary::currentIndexChanged (const QModelIndex& index, const QModelIndex&)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerLibrary::currentIndexChanged\n";
#endif
  if ( m_popup_menu_shown || KPlayerEngine::engine() -> settings() -> rightButton() )
    return;
  history() -> store (listView());
  listView() -> setRootIndex (listView() -> model() -> indexFromIndex (index));
  history() -> apply (listView());
  treeView() -> setRootIsDecorated (true);
}

void KPlayerLibrary::listIndexActivated (const QModelIndex& index)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerNodeView::listIndexActivated\n";
#endif
  QModelIndex idx (treeView() -> model() -> indexFromIndex (index));
  if ( ! idx.isValid() )
    controller() -> play();
  else if ( listView() -> model() -> expandable() )
    listView() -> setExpanded (index, ! listView() -> isExpanded (index));
  else
    treeView() -> setActiveIndex (idx);
}

void KPlayerLibrary::resetActiveIndex (void)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerLibrary::resetActiveIndex\n";
#endif
  treeView() -> setActiveIndex (treeView() -> model() -> indexFromIndex (listView() -> rootIndex()));
}

QSize KPlayerLibrary::sizeHint (void) const
{
  return QSize (QSplitter::sizeHint().width(), m_height);
}

void KPlayerLibrary::resizeEvent (QResizeEvent* event)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Library::resizeEvent " << event -> oldSize().width() << "x" << event -> oldSize().height() << " -> " << event -> size().width() << "x" << event -> size().height() << "\n";
#endif
  QSplitter::resizeEvent (event);
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << " Mouse down " << KPlayerEngine::engine() -> settings() -> anyButton() << "\n";
  kdDebugTime() << " Docked " << parent() -> docked() << "\n";
  kdDebugTime() << " Visible " << parent() -> visible() << "\n";
#endif
  if ( KPlayerEngine::engine() -> settings() -> anyButton() && ! KPlayerEngine::engine() -> layoutInProgress()
    && parent() -> visible() )
  {
    rememberHeight();
#ifdef DEBUG_KPLAYER_NODEVIEW
    kdDebugTime() << " Height " << m_height << "\n";
#endif
  }
}

void KPlayerLibrary::focusInEvent (QFocusEvent* event)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Library getting focus\n";
#endif
  QSplitter::focusInEvent (event);
  activeView() -> setFocus();
}

void KPlayerLibrary::focusOutEvent (QFocusEvent* event)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Library loosing focus\n";
#endif
  QSplitter::focusOutEvent (event);
}

void KPlayerLibrary::open (KPlayerNode* node)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "KPlayerLibrary::open\n";
  if ( node )
    kdDebugTime() << " Node   " << node -> url().url() << "\n";
#endif
  QModelIndex index (treeView() -> model() -> indexForNode (node));
  if ( index.isValid() )
  {
    emit makeVisible();
    treeView() -> setActiveIndex (index);
  }
}

KPlayerLibraryWindow::KPlayerLibraryWindow (KActionCollection* ac, KPlayerPlaylist* playlist, QWidget* parent)
  : QDockWidget (parent)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Creating library window\n";
#endif
  hide();
  setObjectName ("library");
  setWidget (new KPlayerLibrary (ac, playlist, this));
  //setResizeEnabled (true);
  //setCloseMode (QDockWidget::Always);
  //setNewLine (true);
  setWindowTitle (i18n("Multimedia Library"));
  setAllowedAreas (Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
  setFeatures (DockWidgetClosable | DockWidgetMovable | DockWidgetFloatable | DockWidgetVerticalTitleBar);
  setWhatsThis (i18n("Multimedia library is a window where that lets you organize your files, streams, devices, manage your playlists, and choose items for playing. It shows various information about your media files and allows you to search and group them and change their properties."));
  connect (this, SIGNAL (visibilityChanged (bool)), SLOT (setVisibility (bool)));
}

void KPlayerLibraryWindow::setVisibility (bool visibility)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Library visibility " << visibility << " docked " << docked() << "\n";
#endif
  m_visibility = visibility;
}

void KPlayerLibraryWindow::moveEvent (QMoveEvent* event)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "LibraryWindow::moveEvent " << event -> oldPos().x() << "x" << event -> oldPos().y() << " -> " << event -> pos().x() << "x" << event -> pos().y() << " " << isHidden() << " " << docked() << "\n";
#endif
  QDockWidget::moveEvent (event);
  if ( ! isHidden() )
    emit moved (docked());
}

void KPlayerLibraryWindow::resizeEvent (QResizeEvent* event)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "LibraryWindow::resizeEvent " << event -> oldSize().width() << "x" << event -> oldSize().height() << " -> " << event -> size().width() << "x" << event -> size().height() << " " << isHidden() << " " << docked() << "\n";
#endif
  QDockWidget::resizeEvent (event);
  if ( ! isHidden() )
    emit resized();
}

void KPlayerLibraryWindow::focusInEvent (QFocusEvent* event)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Library window getting focus\n";
#endif
  QDockWidget::focusInEvent (event);
  library() -> setFocus();
}

void KPlayerLibraryWindow::focusOutEvent (QFocusEvent* event)
{
#ifdef DEBUG_KPLAYER_NODEVIEW
  kdDebugTime() << "Library window loosing focus\n";
#endif
  QDockWidget::focusOutEvent (event);
}
